#include "usart.h"
#include <util/delay.h>

/*
 * Functie de initializare a controllerului USART
 */
void USART0_init() 
{
	// set pins
	DDRD |= (1 << PD1);
	DDRD &= ~(1 << PD0);
	
    /* seteaza baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 0x0008;

    /* porneste transmitatorul */
	UCSR0B = 0;
    UCSR0B = (1<<TXEN0) | (1<<RXEN0);// | (1<<RXCIE0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop, fara paritate */
    UCSR0C = (1<<UCSZ00) | (1<<UCSZ01) | (1<<USBS0);
	
	USART0_print("$$$");
	_delay_ms(500);
	
	USART0_print("U,9600,N");
    _delay_ms(500);
    USART0_print("SN,Sensor Network\n");
	UBRR0 = 0x0067;    
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 */
void USART0_transmit(char data) 
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
}

/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART0_receive() 
{
    /* asteapta cat timp bufferul e gol */
    //while(!(UCSR0A & (1<<RXC0)));
	
	if (!(UCSR0A & (1<<RXC0)))
		return 'q';

    /* returneaza datele din buffer */
    return UDR0;
}

/*
 * Functie ce transmite un sir de caractere prin USART
 *
 * @param data - sirul (terminat cu '\0') de transmis
 */
void USART0_print(char *data) 
{
    while(*data != '\0')
        USART0_transmit(*data++);
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 * @param stream - nefolosit
 *
 * @return - intotdeauna returneaza 0
 */
int USART0_printf(char data, FILE *stream)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
    return 0;
}
